ByChunks <- function(step_fun, cube_headers, ..., chunks = 'auto',
                     ncores = 1, cluster = NULL, ecflow_output_dir = NULL, 
                     is_ecflow_output_dir_shared = FALSE, 
                     ecflow_host = NULL, silent = FALSE, debug = FALSE,
                     wait = TRUE) {
  MergeArrays <- startR:::.MergeArrays

  # Check input headers
  if ('startR_header' %in% class(cube_headers)) {
    cube_headers <- list(cube_headers)
  }
  if (!all(sapply(lapply(cube_headers, class), 
                  function(x) 'startR_header' %in% x))) {
    stop("All objects passed in 'cube_headers' must be of class 'startR_header', ",
         "as returned by Start().")
  }

  # Check step_fun
  if (!is.function(step_fun)) {
    stop("Parameter 'step_fun' must be a function.")
  }

  # Check cores
  if (!is.numeric(ncores)) {
    stop("Parameter 'ncores' must be a numeric value.")
  }
  ncores <- round(ncores)

  # Check cluster
  on_cluster <- !is.null(cluster)
  default_cluster <- list(queue_host = NULL,
                          queue_type = 'slurm',
                          init_commands = list(''),
                          r_module = 'R',
                          node_memory = NULL,
                          cores_per_job = 1,
                          job_wallclock = '01:00:00',
                          max_jobs = 6,
                          extra_queue_params = list(''))
  if (on_cluster) {
    if (!is.list(cluster)) {
      stop("Parameter 'cluster' must be a named list.")
    }
    if (is.null(names(cluster))) {
      stop("Parameter 'cluster' must be a named list.")
    }
    if (any(!(names(cluster) %in% c('queue_host', 'queue_type', 'init_commands', 
                                    'r_module', 'node_memory', 'cores_per_job', 
                                    'job_wallclock', 'max_jobs', 'extra_queue_params')))) {
      stop("Found invalid component names in parameter 'cluster'.")
    }
    default_cluster[names(cluster)] <- cluster
  }
  localhost_name <- NULL
  cluster <- default_cluster
  if (on_cluster) {
    #localhost_name <- Sys.info()[['nodename']]
    localhost_name <- system('hostname -f', intern = TRUE)
    if (Sys.which('ecflow_client') == '') {
      stop("ecFlow must be installed in order to run the computation on clusters.")
    }
    if (is.null(cluster[['queue_host']])) {
      queue_host <- localhost_name
    } else if ((cluster[['queue_host']] %in% c('localhost', '127.0.0.1', localhost_name)) ||
               grepl(paste0('^', localhost_name), cluster[['queue_host']])) { 
      queue_host <- localhost_name
    }
    if (!(cluster[['queue_type']] %in% c('slurm', 'pbs'))) {
      stop("The only supported 'queue_type's are 'slurm' and 'pbs'.")
    }
    if (!is.list(cluster[['init_commands']]) || 
        !all(sapply(cluster[['init_commands']], is.character))) {
      stop("The component 'init_commands' of the parameter 'cluster' must be a list of ",
           "character strings.")
    }
    if (!is.character(cluster[['r_module']])) {
      stop("The component 'r_module' of the parameter 'cluster' must be a character string.")
    }
    if ((nchar(cluster[['r_module']]) < 1) || (grepl(' ', cluster[['r_module']]))) {
      stop("The component 'r_module' of the parameter 'cluster' must have at least one character ",
           "and contain no blank spaces.")
    }
    if (!is.list(cluster[['extra_queue_params']]) || 
        !all(sapply(cluster[['extra_queue_params']], is.character))) {
      stop("The component 'extra_queue_params' of the parameter 'cluster' must be a list of ",
           "character strings.")
    }
  }

  # Check ecflow_output_dir
  suite_id <- sample(10 ^ 10, 1)
  ecflow_output_dir_suite <- ''
  if (on_cluster) {
    ecflow_output_dir_suite <- paste0(ecflow_output_dir, '/STARTR_CHUNKING_', suite_id, '/')
    dir.create(ecflow_output_dir_suite, recursive = TRUE)
    if (!dir.exists(ecflow_output_dir_suite)) {
      stop("Could not find or create the directory in ",
           "parameter 'ecflow_output_dir'.")
    }
  }

  # Check is_ecflow_output_dir_shared
  if (!is.logical(is_ecflow_output_dir_shared) ||
      (length(is_ecflow_output_dir_shared) < 1)) {
    stop("Parameter 'is_ecflow_output_dir_shared' must be TRUE or FALSE.")
  }

  # Check silent
  if (!is.logical(silent)) {
    stop("Parameter 'silent' must be logical.")
  }

  # Check debug
  if (!is.logical(debug)) {
    stop("Parameter 'debug' must be logical.")
  }
  if (silent) {
    debug <- FALSE
  }

  # Check wait
  if (!is.logical(wait)) {
    stop("Parameter 'wait' must be logical.")
  }

  # Work out chunked dimensions and target dimensions
  all_dims <- lapply(cube_headers, attr, 'Dimensions')
  all_dims_merged <- NULL
  for (i in all_dims) {
    if (is.null(all_dims_merged)) {
      all_dims_merged <- i
    } else {
      all_dims_merged <- startR:::.MergeArrayDims(all_dims_merged, i)[[1]]
    }
  }
  all_dimnames <- names(all_dims_merged)

  target_dims_indices <- which(all_dimnames %in% unlist(attr(step_fun, 'TargetDims')))
  target_dims <- NULL
  if (length(target_dims_indices) > 0) {
    target_dims <- all_dimnames[target_dims_indices]
  }

  chunked_dims <- all_dimnames
  if (length(target_dims_indices) > 0) {
    chunked_dims <- chunked_dims[-target_dims_indices]
  }
  if (length(chunked_dims) < 1) {
    stop("Not possible to process input by chunks. All input dimensions are ",
         "target dimensions.")
  }

  if (length(cube_headers) != length(attr(step_fun, 'TargetDims'))) {
    stop("Number of inputs in parameter 'cube_headers' must be equal to the ",
         "number of inputs expected by the function 'step_fun'.")
  }
  # Check all input headers have matching dimensions
  cube_index <- 1
  for (cube_header in cube_headers) {
    if (!all(attr(cube_header, 'Dimensions') == all_dims_merged[names(attr(cube_header, 'Dimensions'))])) {
      stop("All provided 'cube_headers' must have matching dimension lengths ",
           "with each other.")
    }
    if (!all(attr(step_fun, 'TargetDims')[[cube_index]] %in% names(attr(cube_header, 'Dimensions')))) {
      stop("All provided 'cube_headers' must contain at least the target dimensions ",
           "expected by 'step_fun'.")
    }
    cube_index <- cube_index + 1
    # work out expected result dimensions
  }

  # Check chunks
  default_chunks <- as.list(rep(1, length(chunked_dims)))
  names(default_chunks) <- chunked_dims
  if (length(chunks) == 1 && chunks == 'auto') {
    chunks <- default_chunks
  }
  if (!is.list(chunks)) {
    stop("Parameter 'chunks' must be a named list or 'auto'.")
  }
  if (is.null(names(chunks))) {
    stop("Parameter 'chunks' must be a named list or 'auto'.")
  }
  if (any(!(names(chunks) %in% chunked_dims))) {
    stop("All names in parameter 'chunks' must be one of the non-target dimensions ",
         "present in the cubes in 'cube_headers'. The target dimensions are ", 
         paste(paste0("'", target_dims, "'"), collapse = ', '), ". The non-target ",
         "dimensions (margins) are ", paste(paste0("'", chunked_dims, "'"), collapse = ', '), ".")
  }
  if (any(!(((unlist(chunks) %% 1) == 0) | (unlist(chunks) == 'all')))) {
    stop("All values in parameter 'chunks' must take a numeric value or 'all'.")
  }
  if (any(unlist(chunks) < 1)) {
    stop("All values in parameter 'chunks' must be >= 1.")
  }
  for (chunk_spec in 1:length(chunks)) {
    if (chunks[[chunk_spec]] > all_dims_merged[names(chunks)[chunk_spec]]) {
      stop("Too many chunks requested for the dimension ", names(chunks)[chunk_spec], 
           ". Maximum allowed is ", all_dims_merged[names(chunks)[chunk_spec]])
    }
  }
  default_chunks[names(chunks)] <- chunks
  chunks <- default_chunks

  # Check step_fun
  if (!('startR_step_fun' %in% class(step_fun))) {
    stop("Parameter 'step_fun' must be of the class 'startR_step_fun', as returned ",
         "by the function Step.")
  }

  # Replace 'all's
  chunks_all <- which(unlist(chunks) == 'all')
  if (length(chunks_all) > 0) {
    chunks[chunks_all] <- all_dims[names(chunks)[chunks_all]] 
  }
  # Mount the ecFlow suite
  if (on_cluster) {
    if (!is_ecflow_output_dir_shared) {
      .message("ATTENTION: Dispatching chunks on a remote server without shared file system.",
               " Make sure passwordless access is properly set in both directions.")
    }

    # Copy load_process_save_chunk.R into shared folder
    chunk_script <- file(system.file('chunking/load_process_save_chunk.R', 
                                     package = 'startR'))
    chunk_script_lines <- readLines(chunk_script)
    close(chunk_script)
    chunk_script_lines <- gsub('^out_dir <- *', paste0('out_dir <- ', 
                               paste(deparse(ecflow_output_dir_suite), collapse = '\n')), chunk_script_lines)
    chunk_script_lines <- gsub('^debug <- *', paste0('debug <- ', paste(deparse(debug), collapse = '\n')), 
                               chunk_script_lines)
    deparsed_calls <- paste0('start_calls <- list(')
    for (cube_header in 1:length(cube_headers)) {
      deparsed_calls <- paste0(deparsed_calls, '\nquote(',
        paste(deparse(cube_headers[[cube_header]]), collapse = '\n'), 
        ')')
      if (cube_header < length(cube_headers)) {
        deparsed_calls <- paste0(deparsed_calls, ', ')
      }
    }
    deparsed_calls <- paste0(deparsed_calls, '\n)')
    chunk_script_lines <- gsub('^start_calls <- *', deparsed_calls, chunk_script_lines)
    chunk_script_lines <- gsub('^start_calls_attrs <- *', paste0('start_calls_attrs <- ', paste(deparse(lapply(cube_headers, attributes)), collapse = '\n')), 
                               chunk_script_lines)
    chunk_script_lines <- gsub('^param_dimnames <- *', paste0('param_dimnames <- ', paste(deparse(chunked_dims), collapse = '\n')), 
                               chunk_script_lines)
    chunk_script_lines <- gsub('^cores_per_job <- *', paste0('cores_per_job <- ', cluster[['cores_per_job']]), 
                               chunk_script_lines)
    chunk_script_lines <- gsub('^fun <- *', paste0('fun <- ', paste(deparse(step_fun), collapse = '\n')), 
                               chunk_script_lines)
    chunk_script_lines <- gsub('^params <- *', paste0('params <- ', paste(deparse(list(...)), collapse = '\n')), 
                               chunk_script_lines)
    writeLines(chunk_script_lines, paste0(ecflow_output_dir_suite, '/load_process_save_chunk.R'))
  
    # Copy Chunk.ecf into shared folder
    chunk_ecf_script <- file(system.file('chunking/Chunk.ecf',
                                         package = 'startR'))
    chunk_ecf_script_lines <- readLines(chunk_ecf_script)
    close(chunk_ecf_script)
    chunk_ecf_script_lines <- gsub('^include_queue_header', 
                                   paste0('%include "./', cluster[['queue_type']], '.h"'),
                                   chunk_ecf_script_lines)
    chunk_ecf_script_lines <- gsub('^init_commands', 
                                   paste0(paste0(cluster[['init_commands']], collapse = '\n'), '\n'),
                                   chunk_ecf_script_lines)
    chunk_ecf_script_lines <- gsub('^module load', 
                                   paste0('module load ', cluster[['r_module']]),
                                   chunk_ecf_script_lines)
    ecf_vars <- paste0('%', as.vector(sapply(chunked_dims,
      function(x) {
        c(toupper(x), paste0(toupper(x), '_N'))
      })), '%')
    if (!is_ecflow_output_dir_shared && (cluster[['queue_host']] != localhost_name)) {
      transfer_back_line <- paste0('\nrsync -rav %ECF_HOME% ', localhost_name, 
                                   ':%ECF_HOME%\nrm -f %ECF_HOME%/', 
                                   paste0('*', paste(ecf_vars[((1:(length(ecf_vars) / 2)) * 2) - 1], collapse = '*'), '*.Rds'))
    } else {
      transfer_back_line <- ''
    } 
    chunk_ecf_script_lines <- gsub('^Rscript load_process_save_chunk.R --args', 
                                   paste0('Rscript load_process_save_chunk.R --args ', paste(ecf_vars, collapse = ' '), transfer_back_line),
                                   chunk_ecf_script_lines)
    writeLines(chunk_ecf_script_lines, paste0(ecflow_output_dir_suite, '/Chunk.ecf'))

    # Copy merge_chunks.R into tmp folder
#    merge_script <- file(system.file('chunking/merge_chunks.R',
#                                     package = 'startR'))
#    merge_script_lines <- readLines(merge_script)
#    close(merge_script)
#    merge_script_lines <- gsub('^shared_dir <- *', paste0('shared_dir <- ', 
#                               paste(deparse(shared_dir_suite), collapse = '\n')), merge_script_lines)
#    writeLines(merge_script_lines, paste0(shared_dir_suite, '/merge_chunks.R'))
  
    # Copy Merge.ecf into tmp folder
    #TODO: Modify chain of parameters sent to r script when merging 
    #chunks progressively
#    merge_ecf_script <- file(system.file('chunking/Merge.ecf',
#                                         package = 'startR'))
#    merge_ecf_script_lines <- readLines(merge_ecf_script)
#    close(merge_ecf_script)
#    writeLines(merge_ecf_script_lines, paste0(shared_dir_suite, '/Merge.ecf')) 

    # Copy queue header into shared folder
    #file.copy(system.file(paste0('chunking/', cluster[['queue_type']], '.h'), package = 'startR'),
    #          ecflow_output_dir_suite)
    chunk_queue_header <- file(system.file(paste0('chunking/', cluster[['queue_type']], '.h'), package = 'startR'))
    chunk_queue_header_lines <- readLines(chunk_queue_header)
    close(chunk_queue_header)
    chunk_queue_header_lines <- gsub('^extra_queue_params', 
                                     paste0(paste0(cluster[['extra_queue_params']], collapse = '\n'), '\n'),
                                     chunk_queue_header_lines)
    writeLines(chunk_queue_header_lines, paste0(ecflow_output_dir_suite, '/', cluster[['queue_type']], '.h'))

    # Copy headers
    file.copy(system.file('chunking/head.h', package = 'startR'),
              ecflow_output_dir_suite)
    file.copy(system.file('chunking/tail.h', package = 'startR'),
              ecflow_output_dir_suite)
  }

  add_line <- function(suite, line, tabs) {
    c(suite, paste0(paste(rep(' ', tabs), collapse = ''), line))
  }
  suite <- NULL
  tabs <- 0
  suite <- add_line(suite, paste0('suite STARTR_CHUNKING_', suite_id), tabs)
  tabs <- tabs + 2
  submit_command <- ''
  if (cluster[['queue_type']] == 'slurm') {
    submit_command <- 'sbatch'
  } else if (cluster[['queue_type']] == 'pbs') {
    submit_command <- 'qsub'
  }
  if (on_cluster) {
    suite <- add_line(suite, paste0("edit HOST '", cluster[['queue_host']], "'"), tabs)
    suite <- add_line(suite, paste0("edit ECF_HOST '", localhost_name, "'"), tabs)
    suite <- add_line(suite, paste0("edit EC_HOST_FULL '", localhost_name, "'"), tabs)
    #missing to modify ECF_HOST with .int.bsc.es, hostname -f? iptools::ip_to_hostname('127.0.0.1')?
    if (!is_ecflow_output_dir_shared && (cluster[['queue_host']] != localhost_name)) {
      suite <- add_line(suite, paste0("edit ECF_JOB_CMD 'rsync -rav %ECF_HOME% %HOST%:%ECF_HOME% ; ssh %HOST% ", 
                                      '"', submit_command, ' %ECF_JOB% > %ECF_JOBOUT% 2>&1 &"', "'"), tabs)
    } else {
      suite <- add_line(suite, paste0("edit ECF_JOB_CMD 'ssh %HOST% ", 
                                      '"', submit_command, ' %ECF_JOB% > %ECF_JOBOUT% 2>&1 &"', "'"), tabs)
    }
  } else {
    suite <- add_line(suite, paste0("edit ECF_JOB_CMD '", submit_command, " %ECF_JOB% > %ECF_JOBOUT% 2>&1 &'"), tabs)
  }
  suite <- add_line(suite, paste0("edit ECF_HOME '", ecflow_output_dir_suite, "'"), tabs)
  suite <- add_line(suite, paste0("edit ECF_FILES '", ecflow_output_dir_suite, "'"), tabs)
  suite <- add_line(suite, paste0("edit ECF_INCLUDE '", ecflow_output_dir_suite, "'"), tabs)
  suite <- add_line(suite, paste0("edit CORES_PER_JOB ", cluster[['cores_per_job']], ""), tabs)
  suite <- add_line(suite, paste0("edit JOB_WALLCLOCK '", cluster[['job_wallclock']], "'"), tabs)
  suite <- add_line(suite, paste0("limit max_jobs ", cluster[['max_jobs']]), tabs)
  suite <- add_line(suite, paste0("inlimit max_jobs"), tabs)
  suite <- add_line(suite, "family computation", tabs)
  tabs <- tabs + 2

  if (!silent) {
    startR:::.message(paste0("Processing chunks... ",
                             "remaining time estimate soon..."))
  }
  time_before_first_chunk <- Sys.time()
  time_after_first_chunk <- NULL

  # Open nested ecFlow families
  for (i in length(chunked_dims):1) {
    suite <- add_line(suite, paste0('family ', chunked_dims[i], '_CHUNK_', 1), tabs)
    tabs <- tabs + 2
    suite <- add_line(suite, paste0('edit ', toupper(chunked_dims[i]), ' ', 1), tabs)
    suite <- add_line(suite, paste0('edit ', toupper(chunked_dims[i]), '_N ', chunks[[chunked_dims[i]]]), tabs)
  }

  # Iterate through chunks
  chunk_array <- array(1:prod(unlist(chunks)), dim = (unlist(chunks)))
  arrays_of_results <- vector('list', length(attr(step_fun, 'OutputDims')))
  names(arrays_of_results) <- names(attr(step_fun, 'OutputDims'))
  for (component in 1:length(arrays_of_results)) {
    arrays_of_results[[component]] <- vector('list', prod((unlist(chunks))))
    dim(arrays_of_results[[component]]) <- (unlist(chunks))
  }
  previous_chunk_indices <- rep(1, length(chunks))
  found_first_result <- FALSE
  for (i in 1:length(chunk_array)) {
    chunk_indices <- which(chunk_array == i, arr.ind = TRUE)[1, ]
    names(chunk_indices) <- names(dim(chunk_array))
    # ADD CHUNK SCRIPT TO SUITE
    families_to_jump <- which(chunk_indices != previous_chunk_indices)
    if (length(families_to_jump) > 0) {
      families_to_jump <- max(families_to_jump)
      # Close ecFlow families
      for (j in 1:families_to_jump) {
        tabs <- tabs - 2
        suite <- add_line(suite, paste0('endfamily'), tabs)
      }
      # Open ecFlow families
      for (j in families_to_jump:1) {
        suite <- add_line(suite, paste0('family ', (chunked_dims)[j], '_CHUNK_', chunk_indices[j]), tabs)
        tabs <- tabs + 2
        suite <- add_line(suite, paste0('edit ', toupper((chunked_dims)[j]), ' ', chunk_indices[j]), tabs)
        suite <- add_line(suite, paste0('edit ', toupper((chunked_dims)[j]), '_N ', chunks[[(chunked_dims)[j]]]), tabs)
      }
    }
    suite <- add_line(suite, "task Chunk", tabs)

    if (!on_cluster) {
      if (!silent) {
        startR:::.message(paste("Loading chunk", i, 
                                "out of", length(chunk_array), "..."))
      }
      data <- vector('list', length(cube_headers))
      for (input in 1:length(data)) {
        start_call <- cube_headers[[input]]
        dims_to_alter <- which(names(attr(start_call, 'Dimensions')) %in% names(chunks))
        names_dims_to_alter <- names(attr(start_call, 'Dimensions'))[dims_to_alter]
        # If any dimension comes from split dimensions
        split_dims <- attr(start_call, 'SplitDims')
        for (k in 1:length(split_dims)) {
          if (any(names(split_dims[[k]]) %in% names_dims_to_alter)) {
            chunks_split_dims <- rep(1, length(split_dims[[k]]))
            names(chunks_split_dims) <- names(split_dims[[k]])
            chunks_indices_split_dims <- chunks_split_dims
            split_dims_to_alter <- which(names(split_dims[[k]]) %in% names_dims_to_alter)
            chunks_split_dims[split_dims_to_alter] <- unlist(chunks[names(split_dims[[k]])[split_dims_to_alter]])
            chunks_indices_split_dims[split_dims_to_alter] <- chunk_indices[names(split_dims[[k]])[split_dims_to_alter]]
            start_call[[names(split_dims)[k]]] <- chunk(chunks_indices_split_dims, chunks_split_dims,
                                                        eval(start_call[[names(split_dims)[k]]]))
            dims_to_alter_to_remove <- which(names_dims_to_alter %in% names(split_dims[[k]]))
            if (length(dims_to_alter_to_remove) > 0) {
              dims_to_alter <- dims_to_alter[-dims_to_alter_to_remove]
              names_dims_to_alter <- names_dims_to_alter[-dims_to_alter_to_remove]
            }
          }
        }
        if (length(dims_to_alter) > 0) {
          for (call_dim in names(attr(start_call, 'Dimensions'))[dims_to_alter]) {
            start_call[[call_dim]] <- chunk(chunk_indices[call_dim], chunks[[call_dim]], 
                                            eval(start_call[[call_dim]]))
          }
        }
        start_call[['silent']] <- !debug
        if (!('num_procs' %in% names(start_call))) {
          start_call[['num_procs']] <- ncores
        }
        data[[input]] <- eval(start_call)
      }
      if (!silent) {
        startR:::.message(paste("Processing..."))
      }
      result <- multiApply::Apply(data, 
                                  target_dims = attr(step_fun, 'TargetDims'), 
                                  AtomicFun = step_fun, ..., 
                                  output_dims = attr(step_fun, 'OutputDims'),
                                  ncores = ncores)
      if (!found_first_result) {
        names(arrays_of_results) <- names(result)
        found_first_result <- TRUE
      }
      for (component in 1:length(result)) {
        arrays_of_results[[component]][[i]] <- result[[component]]
      }
      rm(data)
      gc()
    }

    # Time estimate
    if (!on_cluster) {
      if (is.null(time_after_first_chunk)) {
        time_after_first_chunk <- Sys.time()
        if (!silent) {
          estimate <- (time_after_first_chunk - 
                       time_before_first_chunk) *
                       (length(chunk_array) - 1)
          units(estimate) <- 'mins'
          startR:::.message(
            paste0("Remaining time estimate (at ", format(time_after_first_chunk), ") ", 
                   "(neglecting merge time): ", format(estimate))
          )
        }
      }
    }
    previous_chunk_indices <- chunk_indices
  }

  # Close nested ecFlow families
  for (i in length(chunked_dims):1) {
    tabs <- tabs - 2
    suite <- add_line(suite, paste0('endfamily'), tabs)
  }

  # Close the ecFlow suite
  tabs <- tabs - 2
  suite <- add_line(suite, paste0('endfamily'), tabs)
#  suite <- add_line(suite, "family merge", tabs)
#  tabs <- tabs + 2
#  suite <- add_line(suite, "trigger computation == complete", tabs)
#  suite <- add_line(suite, "edit ECF_JOB_CMD 'bash %ECF_JOB% > %ECF_JOBOUT% 2>&1 &'", tabs)
#  suite <- add_line(suite, "task Merge", tabs)
#  tabs <- tabs - 2
#  suite <- add_line(suite, paste0('endfamily'), tabs)

  tabs <- tabs - 2
  suite <- add_line(suite, "endsuite", tabs)

  # Run ecFlow suite if needed
  if (on_cluster) {
    suite_file <- paste0(ecflow_output_dir_suite, '/startR_chunking.def')
    suite_file_o <- file(suite_file)
    writeLines(suite, suite_file_o)
    close(suite_file_o)

    default_ecflow_host <- list(name = 'localhost', port = '5678')
    if (is.null(ecflow_host)) {
      .warning("Parameter 'ecflow_host' has not been specified but execution on ",
               "cluster has been requested. An ecFlow server instance will ",
               "be created on localhost:5678.")
    } else {
      default_ecflow_host[names(ecflow_host)] <- ecflow_host
    }
    ecflow_host <- default_ecflow_host
    system(paste0("ecflow_start.sh -p ", ecflow_host[['port']]))
    system(paste0("ecflow_client --load=", suite_file, " --host=",
                  ecflow_host[['name']], " --port=", ecflow_host[['port']]))
    system(paste0("ecflow_client --begin=STARTR_CHUNKING_", suite_id, 
                  " --host=", ecflow_host[['name']], " --port=", 
                  ecflow_host[['port']]))
     
    startr_exec <- list(cluster = cluster, ecflow_host = ecflow_host, 
                        ecflow_output_dir = ecflow_output_dir, suite_id = suite_id)
    class(startr_exec) <- 'startR_exec'
    if (wait) {
      if (!silent) {
        while (is.null(time_after_first_chunk)) {
          if (any(grepl('.*\\.Rds$', list.files(ecflow_output_dir_suite)))) {
            time_after_first_chunk <- Sys.time()
            estimate <- (time_after_first_chunk - 
                         time_before_first_chunk) * 
                        ceiling((prod(unlist(chunks)) - cluster[['max_jobs']]) / 
                                cluster[['max_jobs']])
            units(estimate) <- 'mins'
            startR:::.message(
              paste0('Remaining time estimate (neglecting queue and ',
                     'merge time) (at ', format(time_after_first_chunk),
                     '): ', format(estimate))
            )
          }
        }
      }
      result <- Collect(startr_exec, wait = TRUE)
      startR:::.message("Computation ended successfully.")
      result
    } else {
      startr_exec
    }
  } else {
    for (component in 1:length(arrays_of_results)) {
      arrays_of_results[[component]] <- startR:::.MergeArrayOfArrays(arrays_of_results[[component]])
    }
    startR:::.message("Computation ended successfully.")
    arrays_of_results
  }
  #TODO: check result dimensions match expected dimensions
}
